#!/usr/bin/perl

# Read in ~/corpora/coca-samples-pos/coca-all, with words, one per line, and POS tags. Create a sliding window of 20 words. Look for
# certain POS sequences, selecting one based on the first command-line argument ($mode). 

$mode = $ARGV[0];
if($mode == 3 || $mode == 8 || $mode == 9 || $mode == 10) {
    $cons_length = 3;
}
else {
    $cons_length = 2;
}

open(INFILE, $ARGV[1]);

$d = 20; # window size
$i = 0; # counts words

while(<INFILE>) {

    chomp($_);

    $_ =~ s/\r//g;   # remove DOS line breaks
	
    $line[$i % $d] = $_;           # Think of the $line[] array as a circle that's filled with the 20 most recent lines (where a line is a word plus tag).
                                   # The most recent line is in slot [$i % $d]. So if we're at word 1007, the 20 lines are in slots 8, 9, 10...6, 7.
    
    for($x=0; $x<$d; $x++) {       # Now update the $t[][] array, in which the most recent line is $t[19][]. We split the line into words: $t[19][0] = word num,
	                           # $t[19][1] = word, $t[19][2] = lemma (not used), $t[19][3] = POS tag.
	@wa = split(/\t/, $line[ ($x + ($i % $d) + 1) % $d ]);        # If we're at word $i = 1007: $line[8] -> $t[0], $line[9] -> $t[1] ..., $line[7] -> $t[19].
	for($w=0; $w<@wa; $w++) {
	    $t[$x][$w] = $wa[$w];
	}
    }
    
    if($i < $d) {
	$i++;
	next;
    }

    # Define $t[9] as the first word of the constituent. For a 2-word constituent, there are nine words on either side (0-8, 11-19). For a 3-word, there are
    # 9 preceding words and 8 following words.
    
    if($mode == 1) {
	# "at nn1" not followed by n* (DT NN)
	if(! ($t[9][3] =~ /^at/ && $t[10][3] =~ /^nn1/ && $t[11][3] !~ /^n/) ) {
	    $i++;
	    next;
	}
    }

    if($mode == 2) {
	# "nnp nnp", not preceded by [adjmn]*, not followed by n* (NNP NNP)
	if(! ($t[8][3] !~ /^[adjmn]/ && $t[9][3] =~ /^np1/ && $t[10][3] =~ /^np1/ && $t[11][3] !~ /^n/) ) {
	    $i++;
	    next;
	}
    }

    if($mode == 3) {
	# "at jj nn1", not followed by n* (DT JJ NN)
	if(! ($t[9][3] =~ /^at/ && $t[10][3] =~ /^jj/ && $t[10][3] !~ /^jj[tr]/ && $t[11][3] =~ /^nn1/ && $t[12][3] !~ /^n/) ) {
	    $i++;
	    next;
	}
    }

    if($mode == 4) {
	# "jj nn2", not preceded by [adjmn] or followed by n* (JJ NNS)
	if(! ($t[8][3] !~ /^[adjmn(rg)]/ && $t[9][3] =~ /^jj/ && $t[9][3] !~ /^jj[tr]/ && $t[10][3] =~ /^nn2/ && $t[11][3] !~ /^n/) ) {
	    $i++;
	    next;
	}
    }

    if($mode == 5) {
	# "at nn2" not followed by n* (DT NNS)
	if(! ($t[9][3] =~ /^at/ && $t[10][3] =~ /^nn2/ && $t[11][3] !~ /^n/) ) {
	    $i++;
	    next;
	}
    }

    if($mode == 6) {
	# "jj nn1", not preceded by [adjmn] or by "jj [c,]", not followed by n* (JJ NN)
	if(! ($t[8][3] !~ /^[adjmn(rg)]/ && (! ($t[7][3] =~ /^jj/ && $t[8][3] =~ /^[(cc)y,]/))
	      && $t[9][3] =~ /^jj/ && $t[9][3] !~ /^jj[tr]/ && $t[10][3] =~ /^nn1/ && $t[11][3] !~ /^n/) ) {
	    $i++;
	    next;
	}
    }

    if($mode == 7) {
	# "nn1 nn2", not preceded by [adjmn]* or by "n* cc", not followed by n* (NN NNS)
	if(! ($t[8][3] !~ /^[adjmn]/ && (! ($t[7][3] =~ /^nn1/ && $t[8][3] =~ /^cc/))
	&& $t[9][3] =~ /^nn1/ && $t[10][3] =~ /^nn2/ && $t[11][3] !~ /^n/) ) {
	    $i++;
	    next;
	}
    }

    if($mode == 8) {
	# "at nn1 nn1" (compare with "at nn2 nn1") (DT NN NN)
	if(! ($t[9][3] =~ /^at/ && $t[10][3] =~ /^nn1/ && $t[11][3] =~ /^nn1/ && $t[12][3] !~ /^n/) ) {
	    $i++;
	    next;
	}
    }

    if($mode == 9) {
	# "The/the jj nn1", not followed by n* (DT JJ NN)
	if(! ($t[9][1] =~ /^[Tt]he/ && $t[10][3] =~ /^jj/ && $t[10][3] !~ /^jj[tr]/ && $t[11][3] =~ /^nn1/ && $t[12][3] !~ /^n/) ) {
	    $i++;
	    next;
	}
    }

    if($mode == 10) {
	# "A/a/An/an jj nn1", not followed by n* (DT JJ NN)
	if(! ($t[9][1] =~ /^(A|a|An|an)$/ && $t[10][3] =~ /^jj/ && $t[10][3] !~ /^jj[tr]/ && $t[11][3] =~ /^nn1/ && $t[12][3] !~ /^n/) ) {
	    $i++;
	    next;
	}
    }

    printf("%d\t", $i-9);
    for($w=0; $w<$d; $w++) {
	if($w == 9) {
	    printf("[ ");
	}
	if($w == 9 + $cons_length) {
	    printf("] ");
	}
	printf("%s ", $t[$w][1]);
    }
    printf("\n");
    
    $i++;
}
